 
//Date: 19/06/2000

/* ************************************** */
/* *   The methodes of class Automata   * */
/* ************************************** */

//constructor
template<class T>
Automata<T>::Automata(string name, int noOfStates, std::vector<string> alpha)
:ITGraph<T>(name,noOfStates)
{  
   this->alphaSet  = alpha;
}

template<class T>
Automata<T>::Automata(const ITGraph<T>& graph,std::vector<string> alpha)
:ITGraph<T>(graph)
{
  this->alphaSet  = alpha;
}

template<class T>
Automata<T>::Automata(const Automata<T>& autoT):ITGraph<T>
(autoT.getName(),0)
{
  std::vector<string> alpha = autoT.getAlphaSet();
  this->reproduct(autoT); 
  this->alphaSet  = alpha;
}

template<class T>
Automata<T>* Automata<T>::reverse(string name)
{
 
  ITGraph<T>* graph = new ITGraph<T>(this->getName());
  graph->reproduct(*this);
  ITGraph<T>* tmp = graph->reverse(graph->getName());
  delete (graph);
  Automata<T> *autoT = new Automata<T>(*tmp,this->getAlphaSet());
  delete tmp;
  autoT->setName(name);
  autoT->setInitialName("i_"+name);
  autoT->setTerminalName("t_"+name);
  return autoT; 
}

template<class T>
int unsigned  Automata<T>::getAlphaSize(void)
{
  return this->alphaSet.size();
}    

template<class T>
std::vector<string> Automata<T>::getAlphaSet(void) const
{
 return this->alphaSet;
}

template<class T>
void Automata<T>::setAlphaSet(std::vector<string>& alpha)
{
  this->alphaSet  = alpha; 
}

template<class T>
void Automata<T>::setAccessible(void)
{
  //set false for all of accessible flags
  std::list<State<T> >::iterator iState, k; 
  std::list<MetaNode<T> >::iterator iNode; 
  std::list<MetaNode<T> > listNodes;

  for(iState=listOfStates.begin();iState!=listOfStates.end();++iState)
     iState->setAccessibleFlag(false);
  //make queue for the reachable states
  std::list<list<State<T> >::iterator> queue;
  
  //for dummy initial state
  listOfStates.begin()->setAccessibleFlag(true); 
  queue.insert(queue.end(), listOfStates.begin()); // begin from state 0  
  while(!queue.empty())
   {  //begin from head of queue
     listNodes = (*queue.begin())->getAllExits();
     for(iNode=listNodes.begin();iNode!=listNodes.end();++iNode)
       { 
         if(iNode->getTransPtr()->getDstPtr()->getAccessibleFlag()==false)
           {   

             iNode->getTransPtr()->getDstPtr()->setAccessibleFlag(true);
             queue.insert(queue.end(),iNode->getTransPtr()->getDstPtr() );  
           }
       } 
     queue.erase(queue.begin());
   } 
}


template<class T>
void Automata<T>::setCoaccessible(void)
{
  //set false for all of coaccessible flags
  std::list<State<T> >::iterator iState,k; 
  std::list<MetaNode<T> >::iterator iNode; 
  std::list<MetaNode<T> > listNodes;
  for(iState=listOfStates.begin();iState!=listOfStates.end();++iState)
     iState->setCoaccessibleFlag(false);
  //make queue for the reachable states
  std::list<list<State<T> >::iterator> queue;
  
  //for dummy terminal state
  k = ++listOfStates.begin();
  k->setCoaccessibleFlag(true);

  queue.insert(queue.end(), k); // begin from state -1
  while(!queue.empty())
   {  //begin from head of queue
     listNodes = (*queue.begin())->getAllEntrances();
     for(iNode=listNodes.begin();iNode!=listNodes.end();++iNode)
       {
         if(iNode->getTransPtr()->getOrgPtr()->getCoaccessibleFlag()==false)
           { 

             iNode->getTransPtr()->getOrgPtr()->setCoaccessibleFlag(true);
             queue.insert(queue.end(),iNode->getTransPtr()->getOrgPtr() );  
           }
       } 
     queue.erase(queue.begin());
   } 
}

template<class T>
void Automata<T>::setDeterministic(void)
{
  std::vector<string> alpha = this->getAlphaSet();
  std::vector<string>::iterator start = alpha.begin();

  //skip empty word
  ++start;

  //  std::list<State<T> > listS 
  std::map<string,string> reached;
  std::list<State<T> >::iterator kState = listOfStates.begin();
  std::list<State<T> >::iterator iState; 
  //skip two states
  ++kState;++kState;
  std::list<MetaNode<T> >::iterator iNode;

  std::list<MetaNode<T> > listNodes;
  // if numberOfInitStates > 1 : non deterministic
  if(this->getNoInitStates() != 1) 
    {
      this->setDetermFlag(0);
      return;
    }

  for(iState=kState;iState!=listOfStates.end();++iState)
    {  //get exits transitions 
      listNodes = iState->getAllExits();
      reached.clear();
      for(iNode=listNodes.begin();iNode!=listNodes.end();++iNode)
	 { 
           T lab = iNode->getTransPtr()->getLabel();
           if(iNode->getTransPtr()->getDstPtr()->getIdOfState()!=-1)
	     {
              string str = lab.getInfixString();
              
              if(find(start,alpha.end(),str) == alpha.end())
              { 
                this->setDetermFlag(0);
                return;
              }

              if(reached.find(str) != reached.end())
              { 
               this->setDetermFlag(0);
               return;
              }
              reached[str] = str;
	     }
         }
    } 
    this->setDetermFlag(1);
    return;
}

template<class T>
void Automata<T>::setCodeterministic(void)
{
  std::vector<string> alpha = this->getAlphaSet();
  std::vector<string>::iterator start = alpha.begin();

  //skip empty word
  ++start;

  std::map<string,string> reached;
  //  std::list<State<T> > listS 
  std::list<State<T> >::iterator kState = listOfStates.begin();
  std::list<State<T> >::iterator iState; 
  //skip two states
  ++kState;++kState;
  std::list<MetaNode<T> >::iterator iNode;
  std::list<MetaNode<T> > listNodes;
  // if numberOfTermStates > 1 : non codeterministic
  if(this->getNoTermStates() != 1) 
    {
      this->setCodetermFlag(0);
      return;
    }
 
  for(iState=kState;iState!=listOfStates.end();++iState)
    {
      //get entrances transitions 
      listNodes = iState->getAllEntrances();
      reached.clear();
      for(iNode=listNodes.begin();iNode!=listNodes.end();++iNode)
	 {
           T lab = iNode->getTransPtr()->getLabel();
           if(iNode->getTransPtr()->getOrgPtr()->getIdOfState()!=0)
	     {
              string str = lab.getInfixString();
              
              //skip empty word
              if(find(start,alpha.end(),str) == alpha.end())
              {
               this->setCodetermFlag(0);
               return;
              }

              if(reached.find(str) != reached.end())
              { 
               this->setCodetermFlag(0);
               return;
              }
              reached[str] =  str;
	     }
         }
    } 
    this->setCodetermFlag(1);
    return;
}

template<class T>
void Automata<T>::setAmbiguous(void)
{
  Automata<T>* tmp = this->productAutos(this,"");
  tmp->accessAutoSelf();
  tmp->coaccessAutoSelf();// create trim automata
  std::list<State<T> >::iterator i;
  i = tmp->listOfStates.begin();
  ++i;++i;
  for(;i!=tmp->listOfStates.end();++i)
    {
     ListOfSymbols his = i->getHistory();
     ListOfSymbols::iterator n = his.begin();
     ListOfSymbols::iterator m = n;
     ++n;
     if(*m!=*n) 
       {
         this->setAmbiguousFlag(1);
         delete tmp;
         return;
       }
    }
  this->setAmbiguousFlag(0);
  delete tmp;
}

template<class T>
void Automata<T>::setComplete(void)
{ 
 int unsigned j,size = this->getAlphaSize();
 std::list<State<T> >::iterator i,k;
 k = listOfStates.begin();++k;++k; 
 std::vector<string> alpha = this->getAlphaSet();
 for(i=k;i!=listOfStates.end();++i)
   {
     //skip empty word
    for(j=1;j<size;++j)
    { 
      std::vector<string> exp = std::vector<string>(0);
      exp.insert(exp.end(),alpha[j]); 
      T lab = T(exp, alpha[0][0]);
      if(!i->isExitByLabel(lab)) 
        {
          setCompleteFlag(0);
          return;
	}
    }
   }
 setCompleteFlag(1);
}

//accessAutoSelf  	
template<class T>
void Automata<T>::accessAutoSelf(void)
{
  std::list<State<T> >::iterator i,k;
  this->setAccessible();
  do
  {
    k = listOfStates.begin();++k;++k; 
    for(i=k;i!=listOfStates.end();++i)
       if(i->getAccessibleFlag()==false) 
         {
           //cout << "\nDelete : " << i->getName() << endl << flush;
           deleteState(i);
           break;
         }
  }while(i!=listOfStates.end());
  this->setNoInitStates();
  this->setNoTermStates();
}

template<class T>
void Automata<T>::coaccessAutoSelf(void)
{
  std::list<State<T> >::iterator i,k;
  this->setCoaccessible();
  do
  {
    k = listOfStates.begin();++k;++k; 
    for(i=k;i!=listOfStates.end();++i)
       if(i->getCoaccessibleFlag()==false) 
         {
           deleteState(i);
           break;
         }
  }while(i!=listOfStates.end());
  this->setNoInitStates();
  this->setNoTermStates();
}

/* ********************************************************* */
/* * Check if the language of the automata is empty or not * */
/* ********************************************************* */

template<class T>
bool Automata<T>::emptyLang(void)
{
   this->setAccessible();
   std::list<State<T> >::iterator k = this->listOfStates.begin();
   ++k;
   if(k->getAccessibleFlag()) return false;
   else return true;
}
  
template<class T>
Automata<T>*  Automata<T>::accessAuto(string name)
{
  
  Automata<T>* tmp = new Automata<T>(this->getName(),0,alphaSet);
  tmp->reproduct(*this);
  tmp->accessAutoSelf();
  tmp->setName(name);
  tmp->setInitialName("i_"+name);
  tmp->setTerminalName("t_"+name);
  return tmp;
}

template<class T>
Automata<T>*  Automata<T>::coaccessAuto(string name)
{
  
  Automata<T>* tmp = new Automata<T>(this->getName(),0,alphaSet);
  tmp->reproduct(*this);
  tmp->coaccessAutoSelf();
  tmp->setName(name);
  tmp->setInitialName("i_"+name);
  tmp->setTerminalName("t_"+name);
  return tmp;
}

template<class T>
void  Automata<T>::trimAuto(string name)
{
  this->accessAutoSelf();
  this->coaccessAutoSelf();
}

template<class T>  
Automata<T>*  Automata<T>::determinize(string nameN)
{
  if(getNoStates()==0) 
    {
      cout << "\nNumber of states is zero, do nothing !" << endl;
      return this;
    }
 
  Automata<T>* tmp = new Automata<T>(nameN,0, alphaSet);
 
  std::map<string, string> newSet;
  std::map<string, int> reachedSets; 
  int  count = 1;
  std::map<string, string>::iterator iMap;

  std::list<std::list<State<T> >::iterator > listIter;
  std::list<std::list<State<T> >::iterator >::iterator iListIter;

  std::list<NameSet<T> > queue;
  std::list<NameSet<T> >::iterator iQueue;
  string name;
  //create initial state
  std::list<MetaNode<T> >::iterator iNode;
  int terminalFlag = 0; 
  std::list<State<T> >::iterator iState = listOfStates.begin();
  std::list<MetaNode<T> > listNodes = iState->getAllExits();

  for(iNode=listNodes.begin();iNode!=listNodes.end();++iNode)
    {  // set terminal flag 
       if(isTerminalState(iNode->getTransPtr()->getDstPtr()->getName())) 
          terminalFlag = 1;
       newSet[iNode->getTransPtr()->getDstPtr()->getName()] = 
              iNode->getTransPtr()->getDstPtr()->getName();  
       listIter.insert(listIter.end(), iNode->getTransPtr()->getDstPtr());  
    }
  //create name with the order 
  
  ListOfSymbols tmpHist;
  name ="{";
  for(iMap=newSet.begin();iMap!=newSet.end();++iMap)
    {
      name = name + iMap->second + ",";
      tmpHist.insert(tmpHist.end(),iMap->second);
    }

  name[name.size() - 1] = '}';
  reachedSets[name] = count++;
  string nameOfState = intToString(reachedSets[name]);
  tmp->addState(nameOfState,tmpHist);
  //update 23/5/2000
  if(terminalFlag == 1)
     tmp->setTerminalState(nameOfState,this->getAlphaSet()[0][0]);  
  //nameOfState is initial
  tmp->setInitialState(nameOfState,this->getAlphaSet()[0][0]);

  std::list<LabelSet<T> > listDstSets;
  std::list<LabelSet<T> >::iterator iSets; 

  NameSet<T> tmpNameSet;
  tmpNameSet.name = name;
  tmpNameSet.states = listIter;
  queue.insert(queue.begin(), tmpNameSet);

  while(!queue.empty())
    { 
    listDstSets = createDstSets( *(queue.begin()) );

    for(iSets=listDstSets.begin();iSets!=listDstSets.end();++iSets)
      { 
        std::list<std::list<State<T> >::iterator>::iterator  iSet;
        terminalFlag = 0; 
        newSet.clear();
        for(iSet=iSets->states.begin();iSet!=iSets->states.end();++iSet)
          {
           // set terminal flag 
           if(isTerminalState((*iSet)->getName()) ) terminalFlag = 1;
           
           newSet[(*iSet)->getName()] = (*iSet)->getName();
          }
 
        tmpHist.clear();
        name ="{";
        tmpHist.clear();
        for(iMap=newSet.begin();iMap!=newSet.end();++iMap)
	      { name = name + iMap->second + ","; 
                tmpHist.insert(tmpHist.end(),iMap->second);
              }
        name[name.size() - 1] = '}';
       
        if(reachedSets.find(name)==reachedSets.end())
	      {
             
                reachedSets[name] = count++;
                nameOfState = intToString(reachedSets[name]);
                State<T> tmpState = State<T>::State(this->maxIdState, 
                                    nameOfState,tmpHist,false,false);
                tmp->addState(&tmpState);
	       
                if(terminalFlag == 1)
                 tmp->setTerminalState(nameOfState,this->getAlphaSet()[0][0]);  
		 
                tmpNameSet.name = name;
                tmpNameSet.states = iSets->states;
                queue.insert(queue.end(), tmpNameSet);
              }
            //add new transition                   
        string name1 = intToString(reachedSets[queue.begin()->name]);
        nameOfState = intToString(reachedSets[name]);
        tmp->addTransition(name1, nameOfState,iSets->label);
      }  
      
    queue.erase(queue.begin());
  }

  ListOfSymbols his;
  his.insert(his.end(), this->getName() );
  tmp->setHistOfITGraph(his);
  tmp->setHistType(DETERMINISTIC_HISTORY);
  tmp->setDetermFlag(1);
  tmp->setNoInitStates();
  tmp->setNoTermStates();
  return tmp;
}
 	
template<class T>  
Automata<T>*  Automata<T>::codeterminize(string name)
{
  Automata<T>* tmp=new Automata<T>(this->getName(),0,
                   this->getAlphaSet());
  tmp->reproduct(*this);
  Automata<T>* tmp1 = tmp->reverse("tmp");
 
  delete (tmp);
  Automata<T>* tmp2 = tmp1->determinize("tmp");

  delete (tmp1);
  
  Automata<T>* tmp3 = tmp2->reverse("tmp");
  
  delete (tmp2);
  
  tmp3->setName(name);
  tmp3->setInitialName("i_"+name);
  tmp3->setTerminalName("t_"+name);
  ListOfSymbols his;
  his.insert(his.end(), this->getName() );
  tmp3->setHistOfITGraph(his);
  tmp3->setHistType(CODETERMINISTIC_HISTORY);
  tmp3->setCodetermFlag(1);
 
  return tmp3;
}

template<class T>
void Automata<T>::completeSelf(void)
{ 
   int unsigned index, size = this->getAlphaSize();
   std::vector<string> alpha = this->getAlphaSet();

   std::list<State<T> >::iterator i,j,z;
   i=this->listOfStates.begin();
 
   if(this->getCompleteFlag()==-1) this->setComplete();
   if(this->getCompleteFlag()==1) return;

   //add state "z"
   int count  = this->getNoStates();
   do
   {
     ++count; 
   }while(isStateByName(intToString(count)));
   this->addState(intToString(count));
   z = this->searchStateByName(intToString(count));
   i=this->listOfStates.begin();++i;++i;
   for(;i!=this->listOfStates.end();++i)
     { 
       //skip empty word
       for(index=1;index<size;++index)
	  {
	      std::vector<string> exp = std::vector<string>(0);
              exp.insert(exp.end(),alpha[index]);  
              T lab = T(exp, alpha[0][0]);
              if(!i->isExitByLabel(lab))              
                 this->addTransition(i,z,lab);
          }
     }

   this->setCompleteFlag(1); 
}

// from  complete and deterministic automaton 
template<class T>	
Automata<T>*  Automata<T>::complement(string name)
{

  std::list<State<T> >::iterator begin,t,end;
  Automata<T>* tmp = new Automata<T>(this->getName(),0,
                     this->getAlphaSet());
  tmp->reproduct(*this);
     
  begin = tmp->listOfStates.begin(); t = begin;++t;
  end = t; ++t;
  std::vector<string> alpha = this->getAlphaSet();
  //not general yet when label of transition is not null ?!
  for(;t!=tmp->listOfStates.end();++t)
    {

      T lab = T(alpha[0][0]);
      if(tmp->isTerminalState(t->getName()))  
          tmp->deleteTransition(t->getName(), end->getName(),lab);
      else
          tmp->addTransition(t->getName(), end->getName(),lab);
    }

  tmp->setName(name);
  tmp->setInitialName("i_"+name);
  tmp->setTerminalName("t_"+name);
  ListOfSymbols his;
  his.insert(his.end(), this->getName() );
  tmp->setHistOfITGraph(his);
  tmp->setHistType(COMPLEMENT_HISTORY);
  tmp->setNoInitStates();
  tmp->setNoTermStates();
  return tmp;
}


template<class T>	
Automata<T>*  Automata<T>::directComp(string name, 
                           std::vector<string>& alpha,bool flag=false)
{
  Automata<T>* tmp = new Automata<T>(this->getName(),0,
                     alpha);
  if(!flag)
      tmp->reproduct(*this);
  else
    {
      tmp->addState("1");
      tmp->setInitialState("1");
      tmp->setTerminalState("1");
    }

  tmp->setName(name);
  tmp->setInitialName("i_"+name);
  tmp->setTerminalName("t_"+name);

  if(tmp->getDetermFlag()==-1) tmp->setDeterministic();
  
  if(tmp->getDetermFlag()==0)
    { 
      Automata<T>* tmpD;     
      tmpD = tmp->determinize("determ");
      delete(tmp);
      if(tmpD->getCompleteFlag()==-1) tmpD->setComplete();
      if(tmpD->getCompleteFlag()==0) tmpD->completeSelf();
      Automata<T>* tmpC = tmpD->complement(name);
      delete(tmpD);
      return tmpC;
    }
 
  if(tmp->getCompleteFlag()==-1) tmp->setComplete();
  if(tmp->getCompleteFlag()==0)  tmp->completeSelf();
  Automata<T>* tmpC = tmp->complement(name);
  delete(tmp); 
  return tmpC;
}



/* ********************************************** */
/* * Create an automata corresponding to a word * */
/* ********************************************** */
       
template<class T>
Automata<T>*  Automata<T>::wordAuto(std::vector<string>& word)
{
  
  std::list<State<T> >::iterator iState;
  std::list<State<T> >::iterator iP;
  int unsigned size = word.size();
  if (size==1&&isEmptyWord(word[0])) 
  {
    Automata<TreeLabel>* tmp=new Automata<TreeLabel>("word_auto",0,
                             this->getAlphaSet());
    tmp->addState("1");
    tmp->setInitialState("1",this->getAlphaSet()[0][0]);
    tmp->setTerminalState("1",this->getAlphaSet()[0][0]);
    tmp->setNoInitStates();
    tmp->setNoTermStates();
    return tmp;
  }
  char k = '0'; 
  
  //It exist a letter of word  that don't belong alphaSet !!!
  Automata<T>* tmp=new Automata<T>("word_auto",size+1,
                   this->getAlphaSet());
  iState=tmp->listOfStates.begin();
  ++iState;++iState;
  for(;iState!=tmp->listOfStates.end();++iState)
    {
      string tmp = "";
      ++k;
      tmp = tmp + k;
      iState->setName(tmp);
    }
  
  tmp->setInitialState(intToString(1),this->getAlphaSet()[0][0]);
  string tmp1 = "";tmp1 = tmp1 + k;
  tmp->setTerminalState(tmp1,this->getAlphaSet()[0][0]);
 
  iState=tmp->listOfStates.begin();
  ++iState;++iState;
  int unsigned  j = 0;
  for(j=0;j<size;++j)
    { 
      iP = iState;++iP;
      std::vector<string> exp = std::vector<string>(0);
      exp.insert(exp.end(),word[j]);
      T lab = T(exp, this->getAlphaSet()[0][0]);
      tmp->addTransition(iState,iP, lab); 
      ++iState;
    }
  tmp->setNoInitStates();
  tmp->setNoTermStates();
  return tmp;
}	

template<class T>
bool Automata<T>::recognize(std::vector<string>& word)
{
  int unsigned  j = 0,size = word.size();
  std::vector<string> alpha = this->getAlphaSet();
  std::vector<string>::iterator start = alpha.begin();
  for(j=0;j<size;++j)
     //include empty word
     if(find(alpha.begin(),alpha.end(), word[j]) == alpha.end()) return false;

  Automata<T>* tmp = wordAuto(word);

  //tmp->display(0);

  Automata<T>* tmp1 = tmp->productAutos(this,"tmp");
  delete(tmp);
  if(tmp1->emptyLang()) { delete(tmp1);return false;}
  else 
  { delete(tmp1);return true;}
  return true;  
}


//third version of product
template<class T>
Automata<T>*  Automata<T>::productAutos(Automata<T>* auto1,string name)
{
  int i,j;
  int count = 1;
  string n1,n2;
  std::list<State<T> >::iterator iState;
  std::list<QueuePos<T> > queue;
  int noLines   = this->getNoStates();noLines += 2;
  int noColumes = auto1->getNoStates();noColumes += 2;
  std::vector<std::list<State<T> >::iterator> thisVector = 
       vector<std::list<State<T> >::iterator>(noLines);
  i=0;
  for(iState=this->listOfStates.begin();
           iState!=this->listOfStates.end();++iState)
    {
      if(i>1) iState->setIdOfState(i);
      thisVector[i++] = iState;
    }

  std::vector<std::list<State<T> >::iterator> auto1Vector = 
       vector<std::list<State<T> >::iterator>(noColumes);
  
  i=0;
  for(iState=auto1->listOfStates.begin();
             iState!=auto1->listOfStates.end();++iState)
    {
       if(i>1) iState->setIdOfState(i); 
       auto1Vector[i++] = iState;
    }

  Automata<T>* autoTmp = new Automata<T>(name, 0, 
                         this->getAlphaSet()); 
  std::list<State<T> >::iterator matrix[noLines][noColumes];
  for(i=0;i<noLines;++i)
  for(j=0;j<noColumes;++j) matrix[i][j] = autoTmp->listOfStates.end();
  
  i=j=0;
  
  QueuePos<T> p = QueuePos<T>(i,j,autoTmp->listOfStates.begin());
  queue.insert(queue.end(),p);
  
  matrix[i][j] = autoTmp->listOfStates.begin();
     
  while(!queue.empty())
    {
       i = queue.begin()->i;
       j = queue.begin()->j;
       std::list<MetaNode<T> >::iterator i1,i2;
       std::list<MetaNode<T> > lnExits = thisVector[i]->getAllExits();
       std::list<MetaNode<T> > clExits = auto1Vector[j]->getAllExits();
       for(i1=lnExits.begin();i1!=lnExits.end();++i1)
       for(i2=clExits.begin();i2!=clExits.end();++i2)
	{ 
          T lab = i1->getTransPtr()->getLabel();
          if(lab==i2->getTransPtr()->getLabel())
	    { 
              std::list<State<T> >::iterator org = queue.begin()->iter;
              std::list<State<T> >::iterator dst;
              int u = i1->getTransPtr()->getDstPtr()->getIdOfState();
              int v = i2->getTransPtr()->getDstPtr()->getIdOfState();
               
              if(u==-1&&v==-1)
                {   
                  std::list<State<T> >::iterator k =
                                        autoTmp->listOfStates.begin();
                  ++k;  
                  //matrix[1][1] = k; 
                  //dst = matrix[1][1];
                  dst = k;
                }
              else if(u==-1||v==-1) continue;
              else
		{
		  
                 if(matrix[u][v]==autoTmp->listOfStates.end()) 
		 { 
                  
                  ListOfSymbols hist;
                  hist.insert(hist.end(),thisVector[u]->getName());
                  hist.insert(hist.end(),auto1Vector[v]->getName());
                  std::list<State<T> >::iterator k = 
                       autoTmp->addState(intToString(count), hist, true);
                  if(k!=autoTmp->listOfStates.end())
		    {
                      count++;
                      matrix[u][v] = k;
                      QueuePos<T> p = QueuePos<T>(u,v,k);
                      queue.insert(queue.end(),p);
                      dst = k;
		    }
                  else    
		    {
                      cout<<"Addition of states is invalid !"<<endl;
                      return NULL; 
		    }
		  }  
                  else dst = matrix[u][v];
		}
	      //cout<<" org= "<<org->getName()<<" dst= "<<
              //dst->getName()<<" lab= "<<lab<<endl;
              autoTmp->addTransition(org,dst,lab,true);
	    }
	}
       queue.erase(queue.begin());
    } 

  autoTmp->setNoInitStates();
  autoTmp->setNoTermStates();
  ListOfSymbols hist;
  hist.insert(hist.end(), this->getName() );
  hist.insert(hist.end(), auto1->getName() );
  autoTmp->setHistOfITGraph(hist);
  autoTmp->setHistType(PRODUCT_HISTORY);
  return autoTmp;
}

template<class T>
Automata<T>*  Automata<T>::minimize(string name)
{
  Automata<T>* tmp=new Automata<T>(this->getName(),0,
                   this->getAlphaSet());
  tmp->reproduct(*this);
  Automata<T>* tmp1 = tmp->codeterminize("");
  delete (tmp);
  Automata<T>* tmp2 = tmp1->determinize("");
  delete (tmp1);
  if(name=="") name = "minimize_" +this->getName();
  tmp2->setName(name);
  tmp2->setInitialName("i_" + tmp2->getName());
  tmp2->setTerminalName("t_" + tmp2->getName());

  ListOfSymbols his;
  his.insert(his.end(), this->getName() );
  tmp2->setHistOfITGraph(his);
  tmp2->setHistType(MINIMAL_HISTORY);
  return tmp2; 
}

template<class T>
Automata<T>*  Automata<T>::delAllSpon(string name)
{
  
   Automata<T>* tmp=new Automata<T>(this->getName(),0,
                   this->getAlphaSet()); 
   tmp->reproduct(*this); 
   //tmp->testMetaNodes();
   tmp->setName(name);
   tmp->setInitialName("i_" + name);
   tmp->setTerminalName("t_" + name);


   //transitive closure
   std::list<State<T> >::iterator iS,kS;
   std::list<MetaNode<T> >::const_iterator i;
  
   std::list<MetaNode<T> > exList;
   std::list<Binome<T> > sponList, tmpList = tmp->getSponList();
   std::list<Binome<T> >::iterator iB;
   
   do
   {
    sponList = tmpList;
    for(iB=sponList.begin();iB!=sponList.end();++iB)
      { 
        exList = iB->dst->getAllExits();
        for(i=exList.begin();i!=exList.end();++i)
	   {
             if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1&&
                i->getTransPtr()->isSpontaneous() )
             if(tmp->addSponTrans(iB->org,i->getTransPtr()->getDstPtr())) 
	       {
                 Binome<T> b;
                 b.org=iB->org; b.dst=i->getTransPtr()->getDstPtr(); 
                 tmpList.insert(tmpList.end(),b) ;
	       } 
	   }
      } 
      
   }while(tmpList.size()>sponList.size());

   //tmp->display();

   //create the other transitions
   for(iB=sponList.begin();iB!=sponList.end();++iB)
      { 
        exList = iB->dst->getAllExits();
        for(i=exList.begin();i!=exList.end();++i)
	   {
             if(!i->getTransPtr()->isSpontaneous()) 
               tmp->addTransition(iB->org,i->getTransPtr()->getDstPtr(),
                                  i->getTransPtr()->getLabel() );
                       
	   }
        //set terminal transition
        if(tmp->isTerminalState(iB->dst->getName())) 
          tmp->setTerminalState(iB->org->getName(),this->getAlphaSet()[0][0]);      
      }

   //delete spontaneous transitions
   for(iB=sponList.begin();iB!=sponList.end();++iB)
      { 
        tmp->deleteSponTrans(iB->org,iB->dst);
      }
 
   ListOfSymbols his;
   his.insert(his.end(), this->getName());
   tmp->setHistOfITGraph(his);
   tmp->setHistType(NO_SPONTANEOUS_HISTORY);
   tmp->setNoInitStates();
   tmp->setNoTermStates();
   return tmp;
}

//for determinization 
template<class T>
std::list<LabelSet<T> >  Automata<T>::createDstSets(const NameSet<T>& ele)
{
 
 std::list<LabelSet<T> > tmp; 
 std::map<T, std::list<std::list<State<T> >::iterator> > tmpMap;
 std::list<std::list<State<T> >::iterator>:: iterator iele;
 std::list<std::list<State<T> >::iterator> listIter = ele.states;


 for(iele=listIter.begin();iele!=listIter.end();++iele)
   { //cout << "loop in creat .." << flush;
     std::list<MetaNode<T> > listNodes = (*iele)->getAllExits();
     std::list<MetaNode<T> >:: iterator iNode;
     for(iNode=listNodes.begin();iNode!=listNodes.end();++iNode)
       {   
           if(iNode->getTransPtr()->getDstPtr()->getIdOfState()!=-1)
           tmpMap[iNode->getTransPtr()->getLabel()].insert(
                 tmpMap[iNode->getTransPtr()->getLabel()].end(),
                 iNode->getTransPtr()->getDstPtr() );
         
       }
   }

 std::map<T, std::list<std::list<State<T> >::iterator> >::iterator iMap;
 for(iMap=tmpMap.begin();iMap!=tmpMap.end();++iMap)
   {
     LabelSet<T> tmp1;
     tmp1.label = iMap->first; 
     tmp1.states = iMap->second;
     tmp.insert(tmp.end(), tmp1);
    
   }
 return tmp;
}

template<class T>
bool Automata<T>::createTuples(std::list<State<T> >::iterator i,
                               std::list<Quadre<T> >& tuples,T& startLab)
{
  bool flag = false;
  std::list<MetaNode<T> > listEx = i->getAllExits();
  std::list<MetaNode<T> >::iterator iEx;
  
  std::list<MetaNode<T> > listEn = i->getAllEntrances();
  std::list<MetaNode<T> >::iterator iEn;
  bool sw = false;
  for(iEn=listEn.begin();iEn!=listEn.end();++iEn)
    { 
     for(iEx=listEx.begin();iEx!=listEx.end();++iEx)
       {
          if(iEn->getTransPtr()->getOrgPtr()==i&&
             iEx->getTransPtr()->getDstPtr()==i)
           {
              startLab = iEn->getTransPtr()->getLabel();
              flag = true;
              std::list<Transition<T> >::iterator mark = iEn->getTransPtr();
              i->deleteEntrance(iEn->getTransPtr());
              i->deleteExit(iEx->getTransPtr());
              this->deleteTransition(mark);
              
              sw = true;
              break; // important !!! 
           }  
       }
      if(sw) break; 
    }

  //this->testMetaNodes();

  listEx = i->getAllExits();
  listEn = i->getAllEntrances();
  for(iEn=listEn.begin();iEn!=listEn.end();++iEn)
     for(iEx=listEx.begin();iEx!=listEx.end();++iEx)
       { 
         Quadre<T> qu;      
         qu.left  = T(iEn->getTransPtr()->getLabel());
         qu.org   = iEn->getTransPtr()->getOrgPtr();
         qu.right = T(iEx->getTransPtr()->getLabel());
         qu.dst   = iEx->getTransPtr()->getDstPtr();  
         tuples.insert(tuples.end(), qu);
       }

   this->ITGraph<T>::deleteState(i);
   return flag;   
}

template<class T>
void Automata<T>::testMetaNodes(void)
{
  std::list<State<T> >::iterator iS;
  std::list<Transition<T> >::iterator iT;
  std::list<MetaNode<T> >::iterator iM;
  std::list<MetaNode<T> > lEx;
  std::list<MetaNode<T> > lEn;
  cout <<"\nTest MetaNode "<<endl<<flush;
  for(iS=listOfStates.begin();iS!=listOfStates.end();++iS)
    { 
     lEx = iS->getAllExits();
     lEn = iS->getAllEntrances();
     T y;
     cout <<"\nNameS :" << iS->getName() << "id= " <<iS->getIdOfState(); 
     for(iM=lEx.begin();iM!=lEx.end();++iM)
       { y = iM->getTransPtr()->getLabel(); 
         cout <<"TransX: "<< y;
       }
     for(iM=lEn.begin();iM!=lEn.end();++iM)
       { y =iM->getTransPtr()->getLabel();
         cout <<"TransE: "<< y;
       }
    }
  for(iT=listOfTrans.begin();iT!=listOfTrans.end();++iT)
    {  
      T lab = iT->getLabel();
      cout<<"label: "<< lab <<", org :"<<iT->getOrgPtr()->getName()<<
	", dst: "<<iT->getDstPtr()->getName()<<endl;
    }
  cout <<"\nTest MetaNode good  "<<endl<<flush;
}

template<class T>
bool Automata<T>::store(string filename)
{
  std::ofstream outAutos(filename.c_str(),ios::out|ios::ate);
  if(!outAutos) 
    {cout<<"Problem of filename or permission !"<<endl;return false;}

  outAutos.seekp(-2,ios::end);// go to SEPARATOR_FILE 
  if (!(outAutos<<this->getName()<<endl)) 
      {   std::cerr << "Write error: s1  !"<<endl;
          outAutos.close();
	  return false; 
      } 

  if (!(outAutos<<this->who()<<endl)) 
      {   std::cerr << "Write error: who  !"<<endl;
          outAutos.close();
	  return false; 
      }
   if (!(outAutos<<this->getId()<<endl))  
      {   std::cerr << "Write error: s2  !"<<endl; 
          outAutos.close();return false; }
   
  
   if (!(outAutos<<this->getDetermFlag()<<endl))  
      {   std::cerr << "Write error: s3  !"<<endl; 
          outAutos.close();return false; }        
 
   if (!(outAutos<<this->getCodetermFlag()<<endl))  
      {   std::cerr << "Write error :s4  !"<<endl; 
          outAutos.close();return false; }
         
   if (!(outAutos<<this->getAmbiguousFlag()<<endl))  
      {   std::cerr << "Write error: s5   !"<<endl; 
          outAutos.close();return false; }     
  
   if (!(outAutos<<this->getCompleteFlag()<<endl))  
      {   std::cerr << "Write error: s   !"<<endl; 
          outAutos.close();return false; }       
  
   if (!(outAutos<<this->getNormalFlag()<<endl))  
      {   std::cerr << "Write error: s7  !"<<endl; 
          outAutos.close();return false; }       
   
   //write alphaset
   if(!(outAutos<<this->getAlphaSize()<<endl)) 
      {   std::cerr << "Write error: s81  !"<<endl; 
          outAutos.close();return false;}
   std::vector<string> alpha = this->getAlphaSet(); 
   for(int unsigned index=0;index<this->getAlphaSize();++index)
   if(!(outAutos<<alpha[index]<<endl)) 
      {   std::cerr << "Write error: s82  !"<<endl; 
          outAutos.close();return false;} 

   if (!(outAutos<<this->getHistType()<<endl))  
      {   std::cerr << "Write error: s9  !"<<endl; 
          outAutos.close();return false; }         
   //cout<<"HIS= "<<this->getHistType() <<endl;

   bool error = false;
   ListOfSymbols l = this->getHistOfITGraph();
   ListOfSymbols::iterator i = l.begin();
   while(i!=l.end())
      {
	if(!(outAutos<<*i<<endl)) 
		{error = true;break;}
              ++i;     
      }
   if(error) {   std::cerr << "Write error s10  !"<<endl; 
                 outAutos.close();return false; }
 
   //mark id end of history
   if(!(outAutos<<""<<endl)) 
   {   std::cerr << "Write error: s11  !"<<endl; 
       outAutos.close();return false; }        
        

   //write states of automaton
   //terminated by id==-2
   error = false;
   
   std::list<State<T> > sL = listOfStates;
   std::list<State<T> >::iterator iS = sL.begin();
   for(;iS!=sL.end();++iS)
    {
      if (!(outAutos<<iS->getIdOfState()<<endl))  
	 {error=true;break;}
      if(!(outAutos<<iS->getName()<<endl)) 
	 {error = true;break;}
      if (!(outAutos<<iS->getAccessibleFlag()<<endl))  
	 {error=true;break;} 
      if (!(outAutos<<iS->getCoaccessibleFlag()<<endl))  
	 {error=true;break;} 
      
      ListOfSymbols hist = iS->getHistory();
      ListOfSymbols::iterator iH = hist.begin();        
      while(iH!=hist.end())
       {
	 if(!(outAutos<<*iH<<endl)) 
	    {error = true;break;}
          ++iH;    
       }
      if(error) {   std::cerr << "Write error: s12  !"<<endl;
                    outAutos.close(); return false; }
 
      //mark od end of history
      if(!(outAutos<<""<<endl)) 
      {   std::cerr << "Write error: s13  !"<<endl; 
          outAutos.close();return false; }
     
   }
   if(error) {   std::cerr << "Write error: s14  !"<<endl; 
                 outAutos.close();return false; }
   
   //mark of end of states
   if (!(outAutos<<-2<<endl))  
	 {   std::cerr << "Write error: s15  !"<<endl; 
             outAutos.close();return false; } 

   //write transitions of automaton
   //terminated by id==NULL(0)
   error = false;
   std::list<Transition<T> >  sT = listOfTrans;
   std::list<Transition<T> >::iterator iT = sT.begin();
   for(;iT!=sT.end();++iT)
    { 
           
         if (!(outAutos<<iT->getIdOfTrans()<<endl))  
	 {error=true;break;}
         if(!(outAutos<<iT->getOrgPtr()->getName()<<endl)) 
		{error = true;break;}    
         if(!(outAutos<<iT->getDstPtr()->getName()<<endl)) 
		{error = true;break;} 
 
         int unsigned infixSize = iT->getLabel().getInfixSize();
	 std::vector<string> lab = iT->getLabel().getInfixSerie();
         if(!(outAutos<<iT->getLabel().getInfixSize()<<endl)) 
		{error = true;break;}  
         for(int unsigned i=0;i<infixSize;++i)
         if(!(outAutos<<lab[i]<<endl)) 
		{error = true;break;}
        
    }
  if(error) {   std::cerr << "Write error: s16  !"<<endl; 
                outAutos.close();return false; } 
  
  //mark of end of transitions
  string separatorA = "";
  char sepa = SEPARATOR_AUTO;
  separatorA = separatorA + sepa;
  if (!(outAutos<<separatorA<<endl))  
     {   std::cerr << "Write error: s17  !"<<endl; 
         outAutos.close();return false; }
  //mark of end of file
  string separatorF = "";
  sepa = SEPARATOR_FILE;
  separatorF = separatorF + sepa;
  if (!(outAutos<<separatorF<<endl))  
     {   std::cerr << "Write error: s18  !"<<endl; 
         outAutos.close();return false; }

  outAutos.close();
  return true;
}

template<class T>   
Automata<T>::Automata(string filename,string name,long pos):
ITGraph<T>(name)
{
  std::ifstream	inAutos(filename.c_str());

  if(!inAutos)
     {
	std::cerr << "Can't open : " << filename << "'\n";
	return;
     }

  inAutos.seekg(pos);
  string separatorF = "";
  char sepa = SEPARATOR_FILE;
  separatorF = separatorF + sepa;
  string separatorA = "";
  sepa = SEPARATOR_AUTO;
  separatorA = separatorA + sepa;
  bool stop = false;
  
  int id; 
  if (!(inAutos>>id ))  
     {std::cerr<<"Read error: id !"<<endl;inAutos.close();return;}
  //skip character end of line
  inAutos.get();
  this->setId(id); 
  //cout <<"iD= "<< id<<endl;

  int deterFlag; 
  if (!(inAutos>>deterFlag))  
	      {std::cerr<<"Read error: deterFlag !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 
  this->setDetermFlag(deterFlag);
  //cout <<"deterflag= "<< deterFlag<<endl;

  int codeterFlag; 
  if (!(inAutos>>codeterFlag))  
	      {std::cerr<<"Read error: CodeterFlag !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 
  this->setCodetermFlag(codeterFlag);   
  //cout <<"codeterflag= "<< codeterFlag<<endl;

  int ambiFlag; 
  if (!(inAutos>>ambiFlag))  
	      {std::cerr<<"Read error: ambiFlag !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 
  this->setAmbiguousFlag(ambiFlag);
  //cout <<"ambiflag= "<< ambiFlag<<endl;

  int comFlag; 
  if (!(inAutos>>comFlag))  
	   {std::cerr<<"Read error:comFlag !"<<endl;return;} 
  //skip character end of line
  inAutos.get(); 
  this->setCompleteFlag(comFlag);
  //cout <<"comflag= "<< comFlag<<endl;

  int norFlag; 
  if (!(inAutos>>norFlag))  
	      {std::cerr<<"Read error: norFlag !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 
  this->setNormalFlag(norFlag);
  //cout <<"norflag= "<< norFlag<<endl;
  
  //read alphaset
  int unsigned sizeAlpha;

  if(!(inAutos>>sizeAlpha))  
	      {std::cerr<<"Read error:sizeAlpha !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 
 
  std::vector<string> alpha = std::vector<string>(sizeAlpha);
  for(int unsigned index=0;index<sizeAlpha;++index)
  if(!std::getline(inAutos, alpha[index])) 
	   {std::cerr<<"Read error: alpha !"<<endl;return;}

  this->setAlphaSet(alpha);  
  //cout <<"alpha= "<<alpha<<endl;

  int hisType; 
  if (!(inAutos>>hisType))  
	      {std::cerr<<"Read error: hisType !"<<endl;return;}
  //skip character end of line
  inAutos.get(); 
  
  this->setHistType(hisType); 
  //cout <<"histype= "<< hisType<<endl;

  //read history of automaton
  //terminated by string NULL 
  ListOfSymbols history;
  string hist;
  bool error = false;
  if(!std::getline(inAutos, hist))
	   {std::cerr<<"Read error: history !"<<endl;return;}
  while(hist!="")
	   {
              history.insert(history.end(),hist);
              //cout<<"\nLoop his !"<<endl; 
              if(!std::getline(inAutos, hist)) 
		{error = true;break;} 
	   }
        
   //cout<<"\nhis OK !"<<endl;
 
  if(error) {std::cerr<<"Read error: history !"<<endl;
             inAutos.close();return;}
  this->setHistOfITGraph(history);

  //read states of automaton
  //terminated by id==-2
  int idState;
  error = false;
  string stateName;
  do
	   {
              if(!(inAutos>>idState))  
		{error=true;break;}   
              if(idState==-2) { inAutos.get();break;}
              //skip character end of line
              inAutos.get(); 
              if(!std::getline(inAutos, stateName)) 
		{error=true;break;}   
              bool accFlag;
              if (!(inAutos>>accFlag))  
		{error=true;break;}   
              inAutos.get(); 
              bool coaccFlag;
              if (!(inAutos>>coaccFlag))  
		 {error=true;break;}   
              inAutos.get(); 
              string hist;
            
              if(!std::getline(inAutos, hist))
	        {std::cerr<<"Read error: history !"<<endl;return;}
              history.clear();
              while(hist!="")
	      {
                history.insert(history.end(),hist);
                //cout<<"\nLoop his !"<<endl; 
                if(!std::getline(inAutos, hist)) 
		   {error = true;break;} 
	      }  

              if(error) break;
              State<T>* state = new State<T>(idState,stateName,history,
                                accFlag, coaccFlag);   
              this->addState(state);
              delete(state);  
   }
   while(1); 
   if(error) {std::cerr<<"Read error: state !"<<endl;
              inAutos.close();return;}

   //read transitions of automaton
   //terminated by id==SEPARATOR_AUTO
        
   error = false;
   do
   { 
              string separatorA = "";
              char sepa = SEPARATOR_AUTO;
              separatorA = separatorA + sepa;
              string idTrans;
              if (!(std::getline(inAutos,idTrans)))
		{error=true;break;}
	      if(idTrans==separatorA) {error = false; break; }
              //cout<<"\nidt = "<<idTrans;

              //skip character end of line
              //inAutos.get();
              string org;
              if(!std::getline(inAutos, org)) 
                {error=true;break;}
              //cout<<"\norg = "<<org;
   
              string dst;
              if(!std::getline(inAutos, dst)) 
		{error=true;break;}   
              //cout<<"\ndst = "<<dst;
             
              int unsigned infixSize;
              if (!(inAutos>>infixSize))  
		 {error=true;break;}   
              inAutos.get(); 

	      std::vector<string> lab =std::vector<string>(infixSize);
              for(int unsigned i=0;i<infixSize;++i)
              if(!std::getline(inAutos, lab[i])) 
		{error=true;break;}
              //cout<<"\nlab = "<<lab;
     
              T label = T(lab,alpha[0][0]);
              this->addTransition(org,dst,label);
	     
   }while(1);
   if(error) {std::cerr<<"Read error:trans !"<<endl;
              inAutos.close();return;}

  inAutos.close();  
  this->setNoInitStates();
  this->setNoTermStates();
}

template<class T> 
void Automata<T>::display(long id)
{
  string choice;          
  if(this->getNoStates() > MAX_STATE_DISPLAY||
     this->getNoTrans()  > MAX_TRANSITION_DISPLAY)
    {
      do
      { 
        cout <<"Automaton is very big !"<<endl;
        cout <<"Number of states = "<<this->getNoStates()<<endl;
        cout <<"Number of transitions = "<<this->getNoTrans()<<endl;
        cout << "Do you want display (y,n) ?";
        cin >> choice;cin.get();
      }
      while(choice!="n"&&choice!="N"&&choice!="y"&&choice!="Y");
      if(choice=="n"||choice=="N") return;
      this->displayBig(id);
    }
  else this->displaySmall();
}

template<class T> 
void Automata<T>::displayBig(long id)
{
  int noCols = getNoColWin(id);
  std::list<State<T> >::iterator iState,kState,tState;
  std::list<MetaNode<T> >::iterator iMetaNode,i;
  std::list<Transition<T> >::iterator iTrans;
  cout << "\n-----DISPLAY----- " << endl; 
  cout << "\nName: " << this->getName() << endl;
  cout<<"Alphabet : ";
  if(this->who()==AUTOMATA||this->who()==GENERAL_AUTOMATA)
    {
      std::vector<string> alpha = this->getAlphaSet();
      cout<<"("<<alpha[0]<<") ";
      for(int unsigned i=1;i<alpha.size();++i) cout<<alpha[i]<<" ";
    }  
  cout<<endl;
  cout << "Number of states: " << this->getNoStates() << endl;
  T result = T();
  kState = this->listOfStates.begin();++kState;++kState;
  int offset=0;
  int count = 0;
  int noStateCol = noCols / STATE_COL_SIZE;
  //cout<<"noStateCol="<<noStateCol<<endl;
  
  for(iState=kState;iState!=this->listOfStates.end(); iState++)
  {     
    cout << iState->getName();
    printChar(' ',STATE_COL_SIZE - iState->getName().size());
    ++offset;
    if(offset>= noStateCol) { offset=0;++count;cout<<endl;}
             if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
  }   
  cout << endl;
  int num;  
  offset=0;
  count = 0;
  num = this->getNoInitStates();
  if(this->getNoStates()==0) num =0;
  cout<<"Number of initial  states: "<< num <<endl;
  if(num>0)
    {
     cout << "Initial  states: " << endl;
     iState = this->listOfStates.begin();
     std::list<MetaNode<T> > listExits = iState->getAllExits();  

     for(iMetaNode=listExits.begin();iMetaNode!=listExits.end(); iMetaNode++)
       {
        if(iMetaNode->getTransPtr()->getDstPtr()->getIdOfState()!=-1) 
        cout<<iMetaNode->getTransPtr()->getDstPtr()->getName();
        printChar(' ',STATE_COL_SIZE -
                  iMetaNode->getTransPtr()->getDstPtr()->getName().size());
        ++offset;
        if(offset>= noStateCol) { offset=0;++count;cout<<endl;}
             if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
       }
    }

  cout << endl; 
  offset=0;
  count = 0;
  num = this->getNoTermStates();
  if(this->getNoStates()==0) num =0;
  cout << "Number of terminal states: " << num << endl;
  if(num>0)
    {
      cout << "Terminal  states: " << endl;
      iState = this->listOfStates.begin();
      ++iState; 
      std::list<MetaNode<T> > listEns = iState->getAllEntrances();
      for(iMetaNode=listEns.begin();iMetaNode!=listEns.end(); iMetaNode++)
	{
         if(iMetaNode->getTransPtr()->getOrgPtr()->getIdOfState()!=0) 
         cout << iMetaNode->getTransPtr()->getOrgPtr()->getName();
         printChar(' ',STATE_COL_SIZE -
                  iMetaNode->getTransPtr()->getOrgPtr()->getName().size());
        ++offset;
        if(offset>= noStateCol) { offset=0;++count;cout<<endl;}
             if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	}
    }

  cout << endl; 
 
  num = this->getNoTrans();
  if(this->getNoStates()==0) num =0;
  cout<<"Number of transitions: "<< num <<endl;
  cout << "Transitions: " << endl;
  if(num>0)
  {
     offset = 0; 
     count  = 0;
     int noTransCol = noCols / TRANS_COL_SIZE;
     //cout<<"noTransCol="<<noTransCol<<endl; 
     iState = this->listOfStates.begin();
     std::list<MetaNode<T> > listExits = iState->getAllExits();  

     for(i=listExits.begin();i!=listExits.end(); i++)
       {
         string dstName = i->getTransPtr()->getDstPtr()->getName();
         T lab = T(i->getTransPtr()->getLabel()); 
         if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1)
	   { 
                                     
             cout<<"--"<<lab<<"->"<<dstName;
             printChar(' ',TRANS_COL_SIZE-
                       (4+dstName.size()+lab.size()));
             ++offset;
             if(offset>=noTransCol) { offset=0;++count;cout<<endl;}
             if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	   }
        
       }
     ++iState;
     kState = iState;
     ++iState;  
     tState = iState; 
     for(;iState!=listOfStates.end();++iState)
       { 
         listExits = iState->getAllExits();  
         for(i=listExits.begin();i!=listExits.end(); ++i)
         {
           string dstName = i->getTransPtr()->getDstPtr()->getName();
           string orgName = i->getTransPtr()->getOrgPtr()->getName();
           if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1&&
              this->isInitialState(iState->getName()))
	     { 
               T lab = T(i->getTransPtr()->getLabel());   
              
               cout<<orgName<<"--"<<lab<<"->"<<dstName;
               printChar(' ',TRANS_COL_SIZE-
               (4+orgName.size()+dstName.size()+lab.size()));
               ++offset;
               if(offset>=noTransCol) { offset=0;++count;cout<<endl;} 
               if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	     }
         }
       }
     iState = tState; 
     for(;iState!=listOfStates.end();++iState)
       { 
         listExits = iState->getAllExits();  
         for(i=listExits.begin();i!=listExits.end(); ++i)
         {
           string dstName = i->getTransPtr()->getDstPtr()->getName();
           string orgName = i->getTransPtr()->getOrgPtr()->getName();
           if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1&&
              !(this->isInitialState(iState->getName())))
	     { 
               T lab = T(i->getTransPtr()->getLabel());
                
               cout<<orgName<<"--"<<lab<<"->"<<dstName;
               printChar(' ',TRANS_COL_SIZE-
               (4+orgName.size()+dstName.size()+lab.size()));
               ++offset;
               if(offset>=noTransCol) { offset=0;++count;cout<<endl;} 
               if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	     }
         }
       }

     iState = kState;;
     listExits = iState->getAllEntrances(); 
      for(i=listExits.begin();i!=listExits.end(); i++)
       {
         string orgName = i->getTransPtr()->getOrgPtr()->getName();
         if(i->getTransPtr()->getOrgPtr()->getIdOfState()==0) orgName = "";
         T lab = T(i->getTransPtr()->getLabel()); 
       
         cout<<orgName<<"--"<<lab<<"->";
         printChar(' ',TRANS_COL_SIZE-
                       (4+orgName.size()+lab.size()));
         ++offset;
         if(offset>=noTransCol) { offset=0;++count;cout<<endl;} 
	 if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
       }
   
  }//if (num>0)
  
  if(this->getNoStates()==0)
    {
     std::list<MetaNode<T> > l = listOfStates.begin()->getAllExits();
     if(l.size()>0)
       {
        T lab = T(l.begin()->getTransPtr()->getLabel());
        if(!lab.empty()) cout<<"Result = "<< lab;
        else  cout<<"Result = "<< this->getAlphaSet()[0];
       }
     else cout<<"Result = null"<<endl; 
    }     

  cout << endl; 
    
}

template<class T> 
void Automata<T>::displaySmall(void)
{ 
   
  std::list<State<T> >::iterator iState,kState,tState;
  std::list<MetaNode<T> >::iterator iMetaNode,i;
  std::list<Transition<T> >::iterator iTrans;
  cout << "\n-----DISPLAY----- " << endl; 
  cout << "\nName: " << this->getName() << endl;
  cout<<"Alphabet : ";
  if(this->who()==AUTOMATA||this->who()==GENERAL_AUTOMATA)
    {
      std::vector<string> alpha = this->getAlphaSet();
      cout<<"("<<alpha[0]<<") ";
      for(int unsigned i=1;i<alpha.size();++i) cout<<alpha[i]<<" ";
    }  
  cout<<endl;
  cout << "Number of states: " << this->getNoStates() << endl;
  T result = T();
  kState = this->listOfStates.begin();++kState;++kState;
  for(iState=kState;iState!=this->listOfStates.end(); iState++)
  
    cout << iState->getName()<<" ";
   

  cout << endl;
  int num;  
  num = this->getNoInitStates();
  if(this->getNoStates()==0) num =0;
  cout<<"Number of initial  states: "<< num <<endl;
  if(num>0)
    {
     cout << "Initial  states: " << endl;
     iState = this->listOfStates.begin();
     std::list<MetaNode<T> > listExits = iState->getAllExits();  

     for(iMetaNode=listExits.begin();iMetaNode!=listExits.end(); iMetaNode++)
       {
        if(iMetaNode->getTransPtr()->getDstPtr()->getIdOfState()!=-1) 
        cout<<iMetaNode->getTransPtr()->getDstPtr()->getName()<<" ";
       }
    }

  cout << endl; 

  num = this->getNoTermStates();
  if(this->getNoStates()==0) num =0;
  cout << "Number of terminal states: " << num << endl;
  if(num>0)
    {
      cout << "Terminal  states: " << endl;
      iState = this->listOfStates.begin();
      ++iState; 
      std::list<MetaNode<T> > listEns = iState->getAllEntrances();
      for(iMetaNode=listEns.begin();iMetaNode!=listEns.end(); iMetaNode++)
	{
         if(iMetaNode->getTransPtr()->getOrgPtr()->getIdOfState()!=0) 
         cout << iMetaNode->getTransPtr()->getOrgPtr()->getName()<<" ";
	}
    }

  cout << endl; 
  int count = 0;
  num = this->getNoTrans();
  if(this->getNoStates()==0) num =0;
  cout<<"Number of transitions: "<< num <<endl;
  cout << "Transitions: " << endl;
  if(num>0)
  {
     iState = this->listOfStates.begin();
     std::list<MetaNode<T> > listExits = iState->getAllExits();  

     for(i=listExits.begin();i!=listExits.end(); i++)
       {
         string dstName = i->getTransPtr()->getDstPtr()->getName();
         T lab = T(i->getTransPtr()->getLabel()); 
         if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1)
	   {    
             cout<<"--"<<lab<<"->"<<dstName<<endl;++count;
             if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	   }
        
       }
     ++iState;
     kState = iState;
     ++iState;  
     tState = iState; 
     for(;iState!=listOfStates.end();++iState)
       { 
         listExits = iState->getAllExits();  
         for(i=listExits.begin();i!=listExits.end(); ++i)
         {
           string dstName = i->getTransPtr()->getDstPtr()->getName();
           string orgName = i->getTransPtr()->getOrgPtr()->getName();
           if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1&&
              this->isInitialState(iState->getName()))
	     { 
               T lab = T(i->getTransPtr()->getLabel()); 
               cout<<orgName<<"--"<<lab<<"->"<<dstName<<endl;++count;
               if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	     }
         }
       }
     iState = tState; 
     for(;iState!=listOfStates.end();++iState)
       { 
         listExits = iState->getAllExits();  
         for(i=listExits.begin();i!=listExits.end(); ++i)
         {
           string dstName = i->getTransPtr()->getDstPtr()->getName();
           string orgName = i->getTransPtr()->getOrgPtr()->getName();
           if(i->getTransPtr()->getDstPtr()->getIdOfState()!=-1&&
              !(this->isInitialState(iState->getName())))
	     { 
               T lab = T(i->getTransPtr()->getLabel()); 
               cout<<orgName<<"--"<<lab<<"->"<<dstName<<endl;++count;
               if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
	     }
         }
       }

     iState = kState;;
     listExits = iState->getAllEntrances(); 
      for(i=listExits.begin();i!=listExits.end(); i++)
       {
         string orgName = i->getTransPtr()->getOrgPtr()->getName();
         if(i->getTransPtr()->getOrgPtr()->getIdOfState()==0) orgName = "";
         T lab = T(i->getTransPtr()->getLabel()); 

         cout<<orgName<<"--"<<lab<<"->"<<endl;++count;
	 if(count>40) { cout<<"\nPress enter !";cin.get();count=0;}  
       }
   
  }//if (num>0)
  
  if(this->getNoStates()==0)
    {
     std::list<MetaNode<T> > l = listOfStates.begin()->getAllExits();
     if(l.size()>0)
       {
        T lab = T(l.begin()->getTransPtr()->getLabel());
        if(!lab.empty()) cout<<"Result = "<< lab;
        else  cout<<"Result = "<< this->getAlphaSet()[0];
       }
     else cout<<"Result = null"; 
    }
     
  cout << endl; 
}

// create a list of spontaneous transitions unless
// initial and terminal spontaneous transitions 
template<class T> 
std::list<Binome<T> > Automata<T>::getSponList(void)
{ 

  std::list<Binome<T> > tmp;
  std::list<Transition<T> >::iterator i;
  for(i=listOfTrans.begin();i!=listOfTrans.end();++i)
  {  
     if(i->getOrgPtr()->getIdOfState()!=0&&
        i->getDstPtr()->getIdOfState()!=-1&&
        i->isSpontaneous() )
       {  
          Binome<T> b;
          b.org=i->getOrgPtr(); b.dst=i->getDstPtr();
          tmp.insert(tmp.end(),b);
       } 
  }
  return tmp;
}
template<class T> 
bool  Automata<T>::isEmptyWord(string& word)
{
  if(word==this->getAlphaSet()[0]) return true;
  else return false;
}
 
template<class T> 
T Automata<T>::getSponLabel(void)
{
  T lab = T(this->getAlphaSet()[0][0]);
  return lab;
}

template<class T> 
bool Automata<T>::deleteSponTrans(std::list<State<T> >::iterator orgPtr, 
                           std::list<State<T> >::iterator dstPtr)
{
  return (deleteTransition(orgPtr,dstPtr,T(this->getAlphaSet()[0][0])));
}
template<class T> 
bool Automata<T>::addSponTrans(std::list<State<T> >::iterator orgPtr, 
                           std::list<State<T> >::iterator dstPtr)
{
  return (this->addTransition(orgPtr,dstPtr, T(this->getAlphaSet()[0][0]) ));
}
